/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			EPETDatabase.cpp
//		DESCRIPTION:	implementation of the CEPETDatabase class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "EPET.h"
#include "EPETDatabase.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//***************************************************************************
//                          CEPETDatabase CLASS
//***************************************************************************

/////////////////////////////////////////////////////////////////////////////
CEPETDatabase::CEPETDatabase(const char* aDatabaseName, 
							 const char* aUserName, 
							 const char* aPassword,
							 bool aReadOnly) :
m_pDBase(NULL),
m_pBuildingsTable(NULL),
m_pSecSystemsTable(NULL),
m_pIsoSysAFMSTable(NULL),
m_pIsoSysSlidingTable(NULL),
m_pIsoSysShearTable(NULL),
m_pEqRealTable(NULL),
m_pEqTimeHistoryTable(NULL),
m_pEqConeTable(NULL),
m_pProjectsTable(NULL)
{
	HRESULT hr = CoCreateInstance( 
		CLSID_DBase, 
		NULL, 
		CLSCTX_INPROC_SERVER, 
		IID_IDBase, 
		(void**)&m_pDBase);

	if (FAILED(hr))
	{
		ASSERT( FALSE );
		m_pDBase->Release();
		m_pDBase = NULL;
	}
	else
	{
		hr = m_pDBase->Initialize(
			A2BSTR(aDatabaseName),
			A2BSTR(aUserName),
			A2BSTR(aPassword),
			aReadOnly ? DB_ACCESS_READ_ONLY : DB_ACCESS_FULL);

		if (FAILED(hr))
		{
			ASSERT( FALSE );
			m_pDBase->Release();
			m_pDBase = NULL;
		}
		else
		{
			UpdateLayout();
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
void CEPETDatabase::UpdateLayout()
{
	USES_CONVERSION;

	bool bCommit = false;
	long lTableID = 0;
	long lFieldID = 0;

	// Cleaup temporary tables

	CComQIPtr<IStdIntArray> pIntArr;
	pIntArr.CoCreateInstance(CLSID_StdIntArray);
	m_pDBase->EnumTables(pIntArr);

	long nCount;
	pIntArr->GetSize(&nCount);

	for (int i = 0; i < nCount; i++)
	{
		long lID;
		pIntArr->Get(i, &lID);

		BSTR bstrName;
		m_pDBase->GetTableNameByID(lID, &bstrName);
		ASSERT(bstrName);

		const char* pszName = OLE2A(bstrName);

		if (pszName[0] == '~')
		{
			m_pDBase->DeleteTable(lID, TRUE);
		}

		SysFreeString(bstrName);
	}

	// Enforce BUILDINGS table

	BSTR bstrTableName = A2BSTR("EPET_BUILDINGS");
	HRESULT hr = m_pDBase->GetTableIDByName(bstrTableName, &lTableID);
	
	if (FAILED(hr))
	{
		m_pDBase->CreateTable(bstrTableName, NULL, &m_pBuildingsTable);
		m_pBuildingsTable->AddField(DB_FIELD_STRING, A2BSTR("DESCRIPTION"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

		CString str;

		for (int i = 0; i < MAX_NUMBER_OF_FLOORS; i++)
		{
			str.Format("MASS%u_KG", i);
			m_pBuildingsTable->AddField(DB_FIELD_FLOAT, A2BSTR(str), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

			str.Format("QUALITY_FACTOR%u", i);
			m_pBuildingsTable->AddField(DB_FIELD_INTEGER, A2BSTR(str), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

			str.Format("ELASTIC_LIMIT%u_CM", i);
			m_pBuildingsTable->AddField(DB_FIELD_FLOAT, A2BSTR(str), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

			str.Format("RIGIDITY_COEFF%u", i);
			m_pBuildingsTable->AddField(DB_FIELD_INTEGER, A2BSTR(str), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		}

		bCommit = true;
	}
	else
	{
		m_pDBase->GetTable(lTableID, &m_pBuildingsTable);
	}

	// Enforce SECONDARY SYSTEMS table

	bstrTableName = A2BSTR("EPET_SEC_SYSTEMS");
	hr = m_pDBase->GetTableIDByName(bstrTableName, &lTableID);
	
	if (FAILED(hr))
	{
		m_pDBase->CreateTable(bstrTableName, NULL, &m_pSecSystemsTable);

		long lRefTableID;
		m_pBuildingsTable->GetID(&lRefTableID);

		m_pSecSystemsTable->AddField(
			DB_FIELD_INTEGER,        // field's type  (ignored)
			A2BSTR("ID_BUILDING"),   // field's name
			NULL,                    // field's description 
			NULL,                    // default value  (ignored)
			FALSE,                   // not a unique value
			lRefTableID,             // ID of the referenced table
			DB_PRIMARY_KEY_FIELD_ID, // reference a primary key
			NULL,                    // the relation name 
			&lFieldID);              // field's ID (set on return)

		m_pSecSystemsTable->AddField(DB_FIELD_STRING, A2BSTR("DESCRIPTION"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pSecSystemsTable->AddField(DB_FIELD_FLOAT, A2BSTR("MASS_KG"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pSecSystemsTable->AddField(DB_FIELD_FLOAT, A2BSTR("DAMPING_RATIO"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pSecSystemsTable->AddField(DB_FIELD_INTEGER, A2BSTR("FLOOR_NUMBER"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

		bCommit = true;
	}
	else
	{
		m_pDBase->GetTable(lTableID, &m_pSecSystemsTable);
	}

	// Enforce ISOLATION SYSTEMS AFMS table

	bstrTableName = A2BSTR("EPET_ISO_SYS_AFMS");
	hr = m_pDBase->GetTableIDByName(bstrTableName, &lTableID);
	
	if (FAILED(hr))
	{
		m_pDBase->CreateTable(bstrTableName, NULL, &m_pIsoSysAFMSTable);

		m_pIsoSysAFMSTable->AddField(DB_FIELD_STRING, A2BSTR("DESCRIPTION"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_INTEGER, A2BSTR("ISO_TYPE"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("NATURAL_FRICTION"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("COEFF_N"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("VRAD0_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("VRAD1_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("VRAD2_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("VRAD3_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("VRAD4_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("VRAD5_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("VRAD6_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("VRAD7_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("VRAD8_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("VRAD9_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("HRAD0_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("HRAD1_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("HRAD2_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("HRAD3_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("HRAD4_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("HRAD5_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("HRAD6_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("HRAD7_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("HRAD8_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysAFMSTable->AddField(DB_FIELD_FLOAT, A2BSTR("HRAD9_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

		bCommit = true;
	}
	else
	{
		m_pDBase->GetTable(lTableID, &m_pIsoSysAFMSTable);
	}

	// Enforce ISOLATION SYSTEMS SLIDING table

	bstrTableName = A2BSTR("EPET_ISO_SYS_SLIDING");
	hr = m_pDBase->GetTableIDByName(bstrTableName, &lTableID);
	
	if (FAILED(hr))
	{
		m_pDBase->CreateTable(bstrTableName, NULL, &m_pIsoSysSlidingTable);

		m_pIsoSysSlidingTable->AddField(DB_FIELD_STRING, A2BSTR("DESCRIPTION"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysSlidingTable->AddField(DB_FIELD_FLOAT, A2BSTR("NATURAL_FRICTION"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysSlidingTable->AddField(DB_FIELD_FLOAT, A2BSTR("RAD_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

		bCommit = true;
	}
	else
	{
		m_pDBase->GetTable(lTableID, &m_pIsoSysSlidingTable);
	}

	// Enforce ISOLATION SYSTEMS SHEAR table

	bstrTableName = A2BSTR("EPET_ISO_SYS_SHEAR");
	hr = m_pDBase->GetTableIDByName(bstrTableName, &lTableID);
	
	if (FAILED(hr))
	{
		m_pDBase->CreateTable(bstrTableName, NULL, &m_pIsoSysShearTable);

		m_pIsoSysShearTable->AddField(DB_FIELD_STRING, A2BSTR("DESCRIPTION"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysShearTable->AddField(DB_FIELD_INTEGER, A2BSTR("ISO_TYPE"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysShearTable->AddField(DB_FIELD_FLOAT, A2BSTR("DAMPING_RATIO"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

		m_pIsoSysShearTable->AddField(DB_FIELD_FLOAT, A2BSTR("COEFF_A"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pIsoSysShearTable->AddField(DB_FIELD_FLOAT, A2BSTR("COEFF_B"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

		bCommit = true;
	}
	else
	{
		m_pDBase->GetTable(lTableID, &m_pIsoSysShearTable);
	}

	// Enforce EARTHQUAKES REAL table

	bstrTableName = A2BSTR("EPET_EQ_REAL");
	hr = m_pDBase->GetTableIDByName(bstrTableName, &lTableID);
	
	if (FAILED(hr))
	{
		m_pDBase->CreateTable(bstrTableName, NULL, &m_pEqRealTable);

		m_pEqRealTable->AddField(DB_FIELD_STRING, A2BSTR("DESCRIPTION"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pEqRealTable->AddField(DB_FIELD_STRING, A2BSTR("EQ_NAME"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pEqRealTable->AddField(DB_FIELD_INTEGER, A2BSTR("EQ_LOCAL_TIME"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pEqRealTable->AddField(DB_FIELD_FLOAT, A2BSTR("EQ_LATITUDE"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pEqRealTable->AddField(DB_FIELD_FLOAT, A2BSTR("EQ_LONGITUDE"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pEqRealTable->AddField(DB_FIELD_FLOAT, A2BSTR("EQ_DEPTH_KM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

		m_pEqRealTable->AddField(DB_FIELD_FLOAT, A2BSTR("MAGNITUDE_ML"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pEqRealTable->AddField(DB_FIELD_FLOAT, A2BSTR("TRANS_NATURAL_PERIOD_SEC"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pEqRealTable->AddField(DB_FIELD_FLOAT, A2BSTR("DAMPING"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pEqRealTable->AddField(DB_FIELD_FLOAT, A2BSTR("SENSITIVITY_CM_G"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pEqRealTable->AddField(DB_FIELD_FLOAT, A2BSTR("TOTAL_TIME_SEC"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

		m_pEqRealTable->AddField(DB_FIELD_STRING, A2BSTR("STATION_NAME"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pEqRealTable->AddField(DB_FIELD_STRING, A2BSTR("STATION_ID"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pEqRealTable->AddField(DB_FIELD_STRING, A2BSTR("ACCEL_ID"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

		bCommit = true;
	}
	else
	{
		m_pDBase->GetTable(lTableID, &m_pEqRealTable);
	}

	// Enforce EARTHQUAKE TIME HISTORY table

	bstrTableName = A2BSTR("EPET_EQ_TIME_HISTORY");
	hr = m_pDBase->GetTableIDByName(bstrTableName, &lTableID);
	
	if (FAILED(hr))
	{
		m_pDBase->CreateTable(bstrTableName, NULL, &m_pEqTimeHistoryTable);

		long lRefTableID;
		m_pEqRealTable->GetID(&lRefTableID);

		m_pEqTimeHistoryTable->AddField(
			DB_FIELD_INTEGER,        // field's type  (ignored)
			A2BSTR("ID_EQ_REAL"),    // field's name
			NULL,                    // field's description 
			NULL,                    // default value  (ignored)
			FALSE,                   // not a unique value
			lRefTableID,             // ID of the referenced table
			DB_PRIMARY_KEY_FIELD_ID, // reference a primary key
			NULL,                    // the relation name 
			&lFieldID);              // field's ID (set on return)

		m_pEqTimeHistoryTable->AddField(DB_FIELD_FLOAT, A2BSTR("TIME_SEC"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pEqTimeHistoryTable->AddField(DB_FIELD_FLOAT, A2BSTR("DISP_CM"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

		bCommit = true;
	}
	else
	{
		m_pDBase->GetTable(lTableID, &m_pEqTimeHistoryTable);
	}

	// Enforce EARTHQUAKES CONE table

	bstrTableName = A2BSTR("EPET_EQ_CONE");
	hr = m_pDBase->GetTableIDByName(bstrTableName, &lTableID);
	
	if (FAILED(hr))
	{
		m_pDBase->CreateTable(bstrTableName, NULL, &m_pEqConeTable);

		m_pEqConeTable->AddField(DB_FIELD_STRING, A2BSTR("DESCRIPTION"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pEqConeTable->AddField(DB_FIELD_FLOAT, A2BSTR("MAX_VEL_CM_SEC"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pEqConeTable->AddField(DB_FIELD_FLOAT, A2BSTR("MIN_PERIOD_SEC"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pEqConeTable->AddField(DB_FIELD_FLOAT, A2BSTR("MAX_PERIOD_SEC"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);
		m_pEqConeTable->AddField(DB_FIELD_FLOAT, A2BSTR("TOTAL_TIME_SEC"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

		bCommit = true;
	}
	else
	{
		m_pDBase->GetTable(lTableID, &m_pEqConeTable);
	}

	// Enforce PROJECTS table

	bstrTableName = A2BSTR("EPET_PROJECTS");
	hr = m_pDBase->GetTableIDByName(bstrTableName, &lTableID);
	
	if (FAILED(hr))
	{
		m_pDBase->CreateTable(bstrTableName, NULL, &m_pProjectsTable);

		m_pProjectsTable->AddField(DB_FIELD_STRING, A2BSTR("DESCRIPTION"), NULL, NULL, FALSE, 0, 0, NULL, &lFieldID);

		long lRefTableID;

		m_pBuildingsTable->GetID(&lRefTableID);
		m_pProjectsTable->AddField(DB_FIELD_INTEGER, A2BSTR("ID_BUILDING"), NULL, NULL, FALSE, lRefTableID, DB_PRIMARY_KEY_FIELD_ID, NULL, &lFieldID);             

		m_pSecSystemsTable->GetID(&lRefTableID);
		m_pProjectsTable->AddField(DB_FIELD_INTEGER, A2BSTR("ID_SEC_SYSTEM"), NULL, NULL, FALSE, lRefTableID, DB_PRIMARY_KEY_FIELD_ID, NULL, &lFieldID);             

		m_pEqRealTable->GetID(&lRefTableID);
		m_pProjectsTable->AddField(DB_FIELD_INTEGER, A2BSTR("ID_EQ_REAL"), NULL, NULL, FALSE, lRefTableID, DB_PRIMARY_KEY_FIELD_ID, NULL, &lFieldID);             

		m_pEqConeTable->GetID(&lRefTableID);
		m_pProjectsTable->AddField(DB_FIELD_INTEGER, A2BSTR("ID_EQ_CONE"), NULL, NULL, FALSE, lRefTableID, DB_PRIMARY_KEY_FIELD_ID, NULL, &lFieldID);             

		m_pIsoSysAFMSTable->GetID(&lRefTableID);
		m_pProjectsTable->AddField(DB_FIELD_INTEGER, A2BSTR("ID_ISO_SYS_AFMS0"), NULL, NULL, FALSE, lRefTableID, DB_PRIMARY_KEY_FIELD_ID, NULL, &lFieldID);             
		m_pProjectsTable->AddField(DB_FIELD_INTEGER, A2BSTR("ID_ISO_SYS_AFMS1"), NULL, NULL, FALSE, lRefTableID, DB_PRIMARY_KEY_FIELD_ID, NULL, &lFieldID);             
		m_pProjectsTable->AddField(DB_FIELD_INTEGER, A2BSTR("ID_ISO_SYS_AFMS2"), NULL, NULL, FALSE, lRefTableID, DB_PRIMARY_KEY_FIELD_ID, NULL, &lFieldID);             
		m_pProjectsTable->AddField(DB_FIELD_INTEGER, A2BSTR("ID_ISO_SYS_AFMS3"), NULL, NULL, FALSE, lRefTableID, DB_PRIMARY_KEY_FIELD_ID, NULL, &lFieldID);             

		m_pIsoSysSlidingTable->GetID(&lRefTableID);
		m_pProjectsTable->AddField(DB_FIELD_INTEGER, A2BSTR("ID_ISO_SYS_SLIDING0"), NULL, NULL, FALSE, lRefTableID, DB_PRIMARY_KEY_FIELD_ID, NULL, &lFieldID);             
		m_pProjectsTable->AddField(DB_FIELD_INTEGER, A2BSTR("ID_ISO_SYS_SLIDING1"), NULL, NULL, FALSE, lRefTableID, DB_PRIMARY_KEY_FIELD_ID, NULL, &lFieldID);             
		m_pProjectsTable->AddField(DB_FIELD_INTEGER, A2BSTR("ID_ISO_SYS_SLIDING2"), NULL, NULL, FALSE, lRefTableID, DB_PRIMARY_KEY_FIELD_ID, NULL, &lFieldID);             
		m_pProjectsTable->AddField(DB_FIELD_INTEGER, A2BSTR("ID_ISO_SYS_SLIDING3"), NULL, NULL, FALSE, lRefTableID, DB_PRIMARY_KEY_FIELD_ID, NULL, &lFieldID);             

		m_pIsoSysShearTable->GetID(&lRefTableID);
		m_pProjectsTable->AddField(DB_FIELD_INTEGER, A2BSTR("ID_ISO_SYS_SHEAR0"), NULL, NULL, FALSE, lRefTableID, DB_PRIMARY_KEY_FIELD_ID, NULL, &lFieldID);             
		m_pProjectsTable->AddField(DB_FIELD_INTEGER, A2BSTR("ID_ISO_SYS_SHEAR1"), NULL, NULL, FALSE, lRefTableID, DB_PRIMARY_KEY_FIELD_ID, NULL, &lFieldID);             
		m_pProjectsTable->AddField(DB_FIELD_INTEGER, A2BSTR("ID_ISO_SYS_SHEAR2"), NULL, NULL, FALSE, lRefTableID, DB_PRIMARY_KEY_FIELD_ID, NULL, &lFieldID);             
		m_pProjectsTable->AddField(DB_FIELD_INTEGER, A2BSTR("ID_ISO_SYS_SHEAR3"), NULL, NULL, FALSE, lRefTableID, DB_PRIMARY_KEY_FIELD_ID, NULL, &lFieldID);             

		bCommit = true;
	}
	else
	{
		m_pDBase->GetTable(lTableID, &m_pProjectsTable);
	}

	// Commit changes

	if (bCommit)
	{
		m_pDBase->Commit();
	}
}

/////////////////////////////////////////////////////////////////////////////
CEPETDatabase::~CEPETDatabase()
{
	if (m_pDBase)
	{
		m_pBuildingsTable->Release();
		m_pSecSystemsTable->Release();
		m_pIsoSysAFMSTable->Release();
		m_pIsoSysSlidingTable->Release();
		m_pIsoSysShearTable->Release();
		m_pEqRealTable->Release();
		m_pEqTimeHistoryTable->Release();
		m_pEqConeTable->Release();
		m_pProjectsTable->Release();

		m_pDBase->Release();
	}
}

/////////////////////////////////////////////////////////////////////////////
IDbRecordSet* CEPETDatabase::TableToRecordSet(IDbTable* aTable)
{
	ASSERT(aTable);

	IDbRecordSet* pRS = NULL;
	HRESULT hr = aTable->QueryInterface(IID_IDbRecordSet, (void**)&pRS);

	if (FAILED(hr))
	{
		ASSERT(FALSE);
		return NULL;
	}

	// Return the interface (no increment)
	pRS->Release();
	return pRS;
}

/////////////////////////////////////////////////////////////////////////////
CString CEPETDatabase::GetFieldAsString(IDbTable* aTable, 
										const char* aFieldName, 
										long aRecordID) const
{
	ASSERT(aTable && aFieldName && aRecordID);

	long lFieldID;
	aTable->GetFieldIDByName(A2BSTR(aFieldName), &lFieldID);

	BSTR bstr;
	CString str;
	CComQIPtr<IDbRecordSet> pRS(aTable);
	pRS->GetFieldAsString(aRecordID, lFieldID, &bstr);

	if (bstr)
	{
		str = bstr;
		SysFreeString(bstr);
	}

	return str;
}

/////////////////////////////////////////////////////////////////////////////
float CEPETDatabase::GetFieldAsFloat(IDbTable* aTable, 
									 const char* aFieldName, 
									 long aRecordID) const
{
	ASSERT(aTable && aFieldName && aRecordID);

	long lFieldID;
	aTable->GetFieldIDByName(A2BSTR(aFieldName), &lFieldID);

	double dValue;
	CComQIPtr<IDbRecordSet> pRS(aTable);
	pRS->GetFieldAsFloat(aRecordID, lFieldID, &dValue);

	return dValue;
}

/////////////////////////////////////////////////////////////////////////////
long CEPETDatabase::GetFieldAsInteger(IDbTable* aTable, 
									  const char* aFieldName, 
									  long aRecordID) const
{
	ASSERT(aTable && aFieldName && aRecordID);

	long lFieldID;
	aTable->GetFieldIDByName(A2BSTR(aFieldName), &lFieldID);

	long lValue;
	CComQIPtr<IDbRecordSet> pRS(aTable);
	pRS->GetFieldAsInteger(aRecordID, lFieldID, &lValue);

	return lValue;
}

/////////////////////////////////////////////////////////////////////////////
bool CEPETDatabase::SetFieldToString(IDbTable* aTable, 
									 const char* aFieldName, 
									 long aRecordID, 
									 const char* aValue) const
{
	ASSERT(aTable && aFieldName && aRecordID);

	long lFieldID;
	aTable->GetFieldIDByName(A2BSTR(aFieldName), &lFieldID);

	CComQIPtr<IDbRecordSet> pRS(aTable);
	HRESULT hr = pRS->SetFieldToString(aRecordID, lFieldID, aValue ? A2BSTR(aValue) : NULL);

	return SUCCEEDED(hr);
}

/////////////////////////////////////////////////////////////////////////////
bool CEPETDatabase::SetFieldToFloat(IDbTable* aTable, 
									const char* aFieldName, 
									long aRecordID, 
									double aValue) const
{
	ASSERT(aTable && aFieldName && aRecordID);

	long lFieldID;
	aTable->GetFieldIDByName(A2BSTR(aFieldName), &lFieldID);

	CComQIPtr<IDbRecordSet> pRS(aTable);
	HRESULT hr = pRS->SetFieldToFloat(aRecordID, lFieldID, aValue);

	return SUCCEEDED(hr);
}

/////////////////////////////////////////////////////////////////////////////
bool CEPETDatabase::SetFieldToInteger(IDbTable* aTable, 
									  const char* aFieldName, 
									  long aRecordID, 
									  long aValue) const

{
	ASSERT(aTable && aFieldName && aRecordID);

	long lFieldID;
	aTable->GetFieldIDByName(A2BSTR(aFieldName), &lFieldID);

	CComQIPtr<IDbRecordSet> pRS(aTable);
	HRESULT hr = pRS->SetFieldToInteger(aRecordID, lFieldID, aValue);

	return SUCCEEDED(hr);
}

//=================================== EOF ===================================
