/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			Earthquake.cpp
//		DESCRIPTION:	implementation of the CEarthquake class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "epet.h"
#include "Earthquake.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//***************************************************************************
//                            CEarthquake CLASS
//***************************************************************************

// Setup/cleanup ************************************************************
/////////////////////////////////////////////////////////////////////////////
CEarthquake::CEarthquake(long aID) :
m_lID(aID),
m_strDescription(""),
m_dStep_SEC(0.01),
m_dTotalTime_SEC(15.0),
m_bCalculated(false)
{
}

/////////////////////////////////////////////////////////////////////////////
CEarthquake::CEarthquake(const CEarthquake& aEq)
{
	m_lID = aEq.m_lID;
	m_strDescription = aEq.m_strDescription;
	m_dStep_SEC = aEq.m_dStep_SEC;
	m_dTotalTime_SEC = aEq.m_dTotalTime_SEC;
	m_bCalculated = aEq.m_bCalculated;

	m_adAbsDisp = aEq.m_adAbsDisp;
	m_adAbsVel = aEq.m_adAbsVel;
	m_adAbsAccel = aEq.m_adAbsAccel;
	m_adAbsAccel2 = aEq.m_adAbsAccel2;
}

/////////////////////////////////////////////////////////////////////////////
const CEarthquake& CEarthquake::operator = (const CEarthquake& aEq)
{
	m_lID = aEq.m_lID;
	m_strDescription = aEq.m_strDescription;
	m_dStep_SEC = aEq.m_dStep_SEC;
	m_dTotalTime_SEC = aEq.m_dTotalTime_SEC;
	m_bCalculated = aEq.m_bCalculated;

	m_adAbsDisp = aEq.m_adAbsDisp;
	m_adAbsVel = aEq.m_adAbsVel;
	m_adAbsAccel = aEq.m_adAbsAccel;
	m_adAbsAccel2 = aEq.m_adAbsAccel2;

	return *this;
}

/////////////////////////////////////////////////////////////////////////////
CEarthquake::~CEarthquake()
{
}

// Get **********************************************************************
/////////////////////////////////////////////////////////////////////////////
double CEarthquake::GetAbsDisp_CM(double aTime) const
{
	int iIndex = TimeToIndex(aTime);

	if (iIndex == -1)
	{
		// Out of range
		return 0.0;
	}

	return m_adAbsDisp[iIndex];
}

/////////////////////////////////////////////////////////////////////////////
double CEarthquake::GetAbsVel_CM_SEC(double aTime) const
{
	int iIndex = TimeToIndex(aTime);

	if (iIndex == -1)
	{
		// Out of range
		return 0.0;
	}

	return m_adAbsVel[iIndex];
}

/////////////////////////////////////////////////////////////////////////////
double CEarthquake::GetAbsAccel_CM_SEC2(double aTime) const
{
	int iIndex = TimeToIndex(aTime);

	if (iIndex == -1)
	{
		// Out of range
		return 0.0;
	}

	return m_adAbsAccel[iIndex];
}

/////////////////////////////////////////////////////////////////////////////
double CEarthquake::GetAbsAccel2_CM_SEC3(double aTime) const
{
	int iIndex = TimeToIndex(aTime);

	if (iIndex == -1)
	{
		// Out of range
		return 0.0;
	}

	return m_adAbsAccel2[iIndex];
}

/////////////////////////////////////////////////////////////////////////////
int CEarthquake::TimeToIndex(double aTime) const
{
	if (!m_dStep_SEC || aTime < 0.0 || aTime > m_dTotalTime_SEC)
	{
		// Undefined step or out of range
		return -1;
	}

	int iIndex = int(aTime / m_dStep_SEC);

	if (iIndex >= m_adAbsDisp.size())
	{
		// Out of range
		return -1;
	}

	return iIndex;
}

// Set **********************************************************************
/////////////////////////////////////////////////////////////////////////////
bool CEarthquake::SetStep_SEC(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_bCalculated = (m_bCalculated && (m_dStep_SEC == aValue));

	m_dStep_SEC = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquake::SetTotalTime_SEC(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_bCalculated = (m_bCalculated && (m_dTotalTime_SEC == aValue));

	m_dTotalTime_SEC = aValue;
	return true;
}


//***************************************************************************
//                          CEarthquakeCone CLASS
//***************************************************************************

// Setup/cleanup ************************************************************
/////////////////////////////////////////////////////////////////////////////
CEarthquakeCone::CEarthquakeCone(long aID) :
CEarthquake(aID),
m_dMaxVelocity_CM_SEC(40.0),
m_dMinPeriod_SEC(0.03),
m_dMaxPeriod_SEC(2.0)
{
	if (m_lID)
	{
		IDbTable* pTable = g_Application.GetDatabase()->GetEqConeTable();
		IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

		long lFieldID;

		BSTR bstrDesc;
		pTable->GetFieldIDByName(A2BSTR("DESCRIPTION"), &lFieldID);
		pRS->GetFieldAsString(m_lID, lFieldID, &bstrDesc);

		if (bstrDesc)
		{
			m_strDescription = bstrDesc;
			SysFreeString(bstrDesc);
		}

		pTable->GetFieldIDByName(A2BSTR("MAX_VEL_CM_SEC"), &lFieldID);
		pRS->GetFieldAsFloat(m_lID, lFieldID, &m_dMaxVelocity_CM_SEC);

		pTable->GetFieldIDByName(A2BSTR("MIN_PERIOD_SEC"), &lFieldID);
		pRS->GetFieldAsFloat(m_lID, lFieldID, &m_dMinPeriod_SEC);

		pTable->GetFieldIDByName(A2BSTR("MAX_PERIOD_SEC"), &lFieldID);
		pRS->GetFieldAsFloat(m_lID, lFieldID, &m_dMaxPeriod_SEC);

		pTable->GetFieldIDByName(A2BSTR("TOTAL_TIME_SEC"), &lFieldID);
		pRS->GetFieldAsFloat(m_lID, lFieldID, &m_dTotalTime_SEC);
	}
}

// Archiving ****************************************************************
/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeCone::Save()
{
	if (!IsValid())
	{
		return false;
	}

	IDbTable* pTable = g_Application.GetDatabase()->GetEqConeTable();
	IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

	if (!m_lID)
	{
		// Create record
		pRS->AddRecord(DB_PRIMARY_KEY_AUTO_GENERATE, &m_lID);
		ASSERT(m_lID);
	}

	long lFieldID;
	pTable->GetFieldIDByName(A2BSTR("DESCRIPTION"), &lFieldID);
	pRS->SetFieldToString(m_lID, lFieldID, m_strDescription.IsEmpty() ? NULL : A2BSTR(m_strDescription));

	pTable->GetFieldIDByName(A2BSTR("MAX_VEL_CM_SEC"), &lFieldID);
	pRS->SetFieldToFloat(m_lID, lFieldID, m_dMaxVelocity_CM_SEC);

	pTable->GetFieldIDByName(A2BSTR("MIN_PERIOD_SEC"), &lFieldID);
	pRS->SetFieldToFloat(m_lID, lFieldID, m_dMinPeriod_SEC);

	pTable->GetFieldIDByName(A2BSTR("MAX_PERIOD_SEC"), &lFieldID);
	pRS->SetFieldToFloat(m_lID, lFieldID, m_dMaxPeriod_SEC);

	pTable->GetFieldIDByName(A2BSTR("TOTAL_TIME_SEC"), &lFieldID);
	pRS->SetFieldToFloat(m_lID, lFieldID, m_dTotalTime_SEC);

	g_Application.GetDatabase()->Commit();

	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeCone::IsValid() const
{
	return (m_dMaxVelocity_CM_SEC > 0.0 && 
	        m_dMinPeriod_SEC > 0.0 && 
			m_dMaxPeriod_SEC > 0.0 && 
			m_dTotalTime_SEC > 0.0 &&
			m_dMinPeriod_SEC < m_dMaxPeriod_SEC);
}

// Get **********************************************************************
/////////////////////////////////////////////////////////////////////////////
CString CEarthquakeCone::GetParamDesc() const
{
	CString str;
	str.Format(
		"Min. period: %.2f sec; Max. period: %.2f sec; Total time: %.2f sec; Max. velocity: %.2f cm/sec", 
		m_dMinPeriod_SEC, 
		m_dMaxPeriod_SEC, 
		m_dTotalTime_SEC, 
		m_dMaxVelocity_CM_SEC);

	return str;
}

// Set **********************************************************************
/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeCone::SetMaxVelocity_CM_SEC(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_bCalculated = (m_bCalculated && (m_dMaxVelocity_CM_SEC == aValue));

	m_dMaxVelocity_CM_SEC = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeCone::SetMinPeriod_SEC(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_bCalculated = (m_bCalculated && (m_dMinPeriod_SEC == aValue));

	m_dMinPeriod_SEC = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeCone::SetMaxPeriod_SEC(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_bCalculated = (m_bCalculated && (m_dMaxPeriod_SEC == aValue));

	m_dMaxPeriod_SEC = aValue;
	return true;
}

// Calculus *****************************************************************
/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeCone::Calculate()
{
	if (m_bCalculated)
		return true;

	CWaitCursor wait;

	int nSteps = int(m_dTotalTime_SEC / m_dStep_SEC) + 1;
	int nExtraSteps = int(m_dMaxPeriod_SEC / m_dStep_SEC);

	m_adAbsDisp.resize  (nSteps + nExtraSteps);
	m_adAbsVel.resize   (nSteps + nExtraSteps);
	m_adAbsAccel.resize (nSteps + nExtraSteps);
	m_adAbsAccel2.resize(nSteps + nExtraSteps);

	// Displacement ---------------------------------------------------------

	// b = tf / (tf - Tf + To)  is the time-periods coefficient
	double b = m_dTotalTime_SEC / (m_dTotalTime_SEC - m_dMaxPeriod_SEC + m_dMinPeriod_SEC);

	// a = (ug)max [1 + (2 PI / ln b) ^ 2] ^ -0.5  is the amplitude of displacement coefficient 
	double a = m_dMaxVelocity_CM_SEC * pow(1 + pow((2 * PI / log(b)), 2.0), -0.5);

	double dTime = 0.0;

	for (int i = 0; i < nSteps + nExtraSteps; i++)
	{
		// ug = a t sin (2 PI logb [(b - 1) t / bTo + 1])
		double ug = a * dTime * sin(2.0 * PI * log((b - 1.0) * dTime / (b * m_dMinPeriod_SEC) + 1.0) / log(b));
		m_adAbsDisp[i] = ug;

		dTime += m_dStep_SEC;

		if (i >= nSteps)
		{
			if (m_adAbsDisp[i] == 0.0 || ((m_adAbsDisp[i - 1] <= 0.0) ^ (m_adAbsDisp[i] <= 0.0)))
			{
				// End with zero displacement (requirement on output)
				m_adAbsDisp[i] = 0.0;
				m_dTotalTime_SEC = dTime;
				nSteps = i + 1;
				break;
			}
		}
	}

	// Velocity -------------------------------------------------------------

	for (i = 1; i < nSteps; i++)
	{
		double dValue = (m_adAbsDisp[i] - m_adAbsDisp[i - 1]) / m_dStep_SEC;
		m_adAbsVel[i] = dValue;
	}

	// Acceleration ---------------------------------------------------------

	for (i = 1; i < nSteps; i++)
	{
		double dValue = (m_adAbsVel[i] - m_adAbsVel[i - 1]) / m_dStep_SEC;
		m_adAbsAccel[i] = dValue;
	}

	// 3rd derivation -------------------------------------------------------

	for (i = 1; i < nSteps; i++)
	{
		double dValue = (m_adAbsAccel[i] - m_adAbsAccel[i - 1]) / m_dStep_SEC;
		m_adAbsAccel2[i] = dValue;
	}

	m_bCalculated = true;
	return true;
}


//***************************************************************************
//                          CEarthquakeReal CLASS
//***************************************************************************

/////////////////////////////////////////////////////////////////////////////
CEarthquakeReal::CEarthquakeReal(long aID) :
CEarthquake(aID),
m_strName(""),
m_lLocalTime(0),
m_dLatitude(0.0),
m_dLongitude(0.0),
m_dDepth_KM(0.0),
m_dMagnitude_ML(0.0),
m_dTransNaturalPeriod_SEC(0.0),
m_dDamping(0.0),
m_dSensitivity_CM_G(0.0),
m_strStationName(""),
m_strStationID(""),
m_strAccelID("")
{
	USES_CONVERSION;

	if (m_lID)
	{
		// Get properties ---------------------------------------------------

		IDbTable* pTable = DB_PTR->GetEqRealTable();

		m_strDescription = DB_PTR->GetFieldAsString(pTable, "DESCRIPTION", m_lID);
		m_strName = DB_PTR->GetFieldAsString(pTable, "EQ_NAME", m_lID);
		m_lLocalTime= DB_PTR->GetFieldAsInteger(pTable, "EQ_LOCAL_TIME", m_lID);
		m_dLatitude = DB_PTR->GetFieldAsFloat(pTable, "EQ_LATITUDE", m_lID);
		m_dLongitude= DB_PTR->GetFieldAsFloat(pTable, "EQ_LONGITUDE", m_lID);
		m_dDepth_KM = DB_PTR->GetFieldAsFloat(pTable, "EQ_DEPTH_KM", m_lID);
		m_dMagnitude_ML = DB_PTR->GetFieldAsFloat(pTable, "MAGNITUDE_ML", m_lID);
		m_dTransNaturalPeriod_SEC = DB_PTR->GetFieldAsFloat(pTable, "TRANS_NATURAL_PERIOD_SEC", m_lID);
		m_dDamping= DB_PTR->GetFieldAsFloat(pTable, "DAMPING", m_lID);
		m_dSensitivity_CM_G = DB_PTR->GetFieldAsFloat(pTable, "SENSITIVITY_CM_G", m_lID);
		m_dTotalTime_SEC= DB_PTR->GetFieldAsFloat(pTable, "TOTAL_TIME_SEC", m_lID);
		m_strStationName= DB_PTR->GetFieldAsString(pTable, "STATION_NAME", m_lID);
		m_strStationID= DB_PTR->GetFieldAsString(pTable, "STATION_ID", m_lID);
		m_strAccelID= DB_PTR->GetFieldAsString(pTable, "ACCEL_ID", m_lID);

		// Get time history -------------------------------------------------

		CString str;
		str.Format("ID_EQ_REAL=%u", m_lID);

		CComPtr<IStdIntArray> pArray;
		pArray.CoCreateInstance(CLSID_StdIntArray);

		pTable = DB_PTR->GetEqTimeHistoryTable();
		IDbRecordSet* pRS = DB_PTR->TableToRecordSet(pTable);
		pRS->EnumRecords(A2OLE(str), DB_PRIMARY_KEY_FIELD_ID, pArray);

		vector<double> times;

		long nIDs;
		pArray->GetSize(&nIDs);

		for (int i = 0; i < nIDs; i++)
		{
			long lID;
			pArray->Get(i, &lID);

			double dTime = DB_PTR->GetFieldAsFloat(pTable, "TIME_SEC", lID);
			double dDisp = DB_PTR->GetFieldAsFloat(pTable, "DISP_CM", lID);

			times.push_back(dTime);
			m_adAbsDisp.push_back(dDisp);
		}

		if (times.size() > 1)
		{
			m_dTotalTime_SEC = times.back();
			m_dStep_SEC = m_dTotalTime_SEC / double(times.size() - 1);
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeReal::Save()
{
	CWaitCursor wait;

	USES_CONVERSION;

	if (!IsValid())
	{
		return false;
	}

	// Set properties -------------------------------------------------------

	IDbTable* pTable = DB_PTR->GetEqRealTable();

	if (!m_lID)
	{
		// Create record
		IDbRecordSet* pRS = DB_PTR->TableToRecordSet(pTable);
		pRS->AddRecord(DB_PRIMARY_KEY_AUTO_GENERATE, &m_lID);
		ASSERT(m_lID);
	}

	DB_PTR->SetFieldToString (pTable, "DESCRIPTION", m_lID, m_strDescription);
	DB_PTR->SetFieldToString (pTable, "EQ_NAME", m_lID, m_strName);
	DB_PTR->SetFieldToInteger(pTable, "EQ_LOCAL_TIME", m_lID, m_lLocalTime);
	DB_PTR->SetFieldToFloat  (pTable, "EQ_LATITUDE", m_lID, m_dLatitude);
	DB_PTR->SetFieldToFloat  (pTable, "EQ_LONGITUDE", m_lID, m_dLongitude);
	DB_PTR->SetFieldToFloat  (pTable, "EQ_DEPTH_KM", m_lID, m_dDepth_KM);
	DB_PTR->SetFieldToFloat  (pTable, "MAGNITUDE_ML", m_lID, m_dMagnitude_ML  );
	DB_PTR->SetFieldToFloat  (pTable, "TRANS_NATURAL_PERIOD_SEC", m_lID, m_dTransNaturalPeriod_SEC);
	DB_PTR->SetFieldToFloat  (pTable, "DAMPING", m_lID, m_dDamping);
	DB_PTR->SetFieldToFloat  (pTable, "SENSITIVITY_CM_G", m_lID, m_dSensitivity_CM_G);
	DB_PTR->SetFieldToFloat  (pTable, "TOTAL_TIME_SEC", m_lID, m_dTotalTime_SEC);
	DB_PTR->SetFieldToString (pTable, "STATION_NAME", m_lID, m_strStationName);
	DB_PTR->SetFieldToString (pTable, "STATION_ID", m_lID, m_strStationID);
	DB_PTR->SetFieldToString (pTable, "ACCEL_ID", m_lID, m_strAccelID );

	// Set time history -----------------------------------------------------

	CString str;
	str.Format("ID_EQ_REAL=%u", m_lID);

	CComPtr<IStdIntArray> pArray;
	pArray.CoCreateInstance(CLSID_StdIntArray);

	pTable = DB_PTR->GetEqTimeHistoryTable();
	IDbRecordSet* pRS = DB_PTR->TableToRecordSet(pTable);
	pRS->EnumRecords(A2OLE(str), DB_PRIMARY_KEY_FIELD_ID, pArray);

	long nIDs, lID;
	pArray->GetSize(&nIDs);

	for (int i = 0; i < nIDs; i++)
	{
		pArray->Get(i, &lID);
		pRS->DeleteRecord(lID, TRUE);
	}

	double dTime = 0.0;

	for (i = 0; i < m_adAbsDisp.size(); i++)
	{
		pRS->AddRecord(DB_PRIMARY_KEY_AUTO_GENERATE, &lID);
		DB_PTR->SetFieldToInteger(pTable, "ID_EQ_REAL", lID, m_lID);
		DB_PTR->SetFieldToFloat(pTable, "TIME_SEC", lID, dTime);
		DB_PTR->SetFieldToFloat(pTable, "DISP_CM", lID, m_adAbsDisp[i]);

		dTime += m_dStep_SEC;
	}

	DB_PTR->Commit();

	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeReal::IsValid() const
{
	return m_adAbsDisp.size() > 1 && m_dTotalTime_SEC > 0.0;
}

/////////////////////////////////////////////////////////////////////////////
CString CEarthquakeReal::GetParamDesc() const
{
	CString str;
	str.Format(
		"Name: %s; Depth: %.2f km; Magnitude: %.2f ml; Total time: %d sec", 
		m_strName, 
		m_dDepth_KM, 
		m_dMagnitude_ML, 
		m_dTotalTime_SEC);

	return str;
}

//****************************** Modifying data *****************************
/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeReal::SetName(const char* aValue)
{
	m_strName = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeReal::SetLocalTime(long aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_lLocalTime = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeReal::SetLatitude(double aValue)
{
	m_dLatitude = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeReal::SetLongitude(double aValue)
{
	m_dLongitude = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeReal::SetDepth_KM(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_dDepth_KM = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeReal::SetMagnitude_ML(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_dMagnitude_ML = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeReal::SetTransNaturalPeriod_SEC(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_dTransNaturalPeriod_SEC = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeReal::SetDamping(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_dDamping = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeReal::SetSensitivity_CM_G(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_dSensitivity_CM_G = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeReal::SetStationName(const char* aValue)
{
	m_strStationName = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeReal::SetStationID(const char* aValue)
{
	m_strStationID = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeReal::SetAccelID(const char* aValue)
{
	m_strAccelID = aValue;
	return true;
}

// Calculus *****************************************************************
/////////////////////////////////////////////////////////////////////////////
bool CEarthquakeReal::Calculate()
{
	if (m_bCalculated)
		return true;

	if (!m_dStep_SEC || !m_dTotalTime_SEC)
		return false;

	CWaitCursor wait;

	int nSteps = m_adAbsDisp.size();

	m_adAbsVel.resize   (nSteps);
	m_adAbsAccel.resize (nSteps);
	m_adAbsAccel2.resize(nSteps);

	// Velocity -------------------------------------------------------------

	for (int i = 1; i < nSteps; i++)
	{
		double dValue = (m_adAbsDisp[i] - m_adAbsDisp[i - 1]) / m_dStep_SEC;
		m_adAbsVel[i] = dValue;
	}

	// Acceleration ---------------------------------------------------------

	for (i = 1; i < nSteps; i++)
	{
		double dValue = (m_adAbsVel[i] - m_adAbsVel[i - 1]) / m_dStep_SEC;
		m_adAbsAccel[i] = dValue;
	}

	// 3rd derivation -------------------------------------------------------

	for (i = 1; i < nSteps; i++)
	{
		double dValue = (m_adAbsAccel[i] - m_adAbsAccel[i - 1]) / m_dStep_SEC;
		m_adAbsAccel2[i] = dValue;
	}

	m_bCalculated = true;
	return true;
}

//=================================== EOF ===================================
