/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			GLFloor.cpp
//		DESCRIPTION:	implementation of the GLFloor class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "epet.h"
#include "GLFloor.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//***************************************************************************
//                             GlFloor CLASS
//***************************************************************************

long GlFloor::m_iList = 0;
CLock GlFloor::m_Lock;

//**************************** Setup/cleanup ********************************
/////////////////////////////////////////////////////////////////////////////
GlFloor::GlFloor(int aFloor, IVolSpace* aSpace, const GlColor& aColor) :
m_iFloor    (aFloor),
m_pCoord    (NULL),
m_pVolObject(NULL),
m_Color     (aColor),
m_bShowText (true),
m_bstrText  (NULL),
m_dwUserData(0),
m_dRadius   (2.0)
{
	// Create 3D object

	HRESULT hr = CoCreateInstance(
		CLSID_VolObject, 
		NULL, 
		CLSCTX_INPROC_SERVER, 
		IID_IVolObject, 
		(void**)&m_pVolObject);

	if (FAILED(hr))
	{
		ASSERT(FALSE);
		return;
	}

	// Create coordinate system

	hr = CoCreateInstance(
		CLSID_GlCoord, 
		NULL, 
		CLSCTX_INPROC_SERVER, 
		IID_IGlCoord, 
		(void**)&m_pCoord);

	if (FAILED(hr))
	{
		ASSERT(FALSE);
		return;
	}

	m_pCoord->SetOrigin3d(0.0, m_iFloor * GetFloorHeight(), 0.0);
	m_pVolObject->AttachCoord(m_pCoord);

	CString str;
	str.Format("M%d", m_iFloor);
	m_bstrText = str.AllocSysString();

	m_pVolObject->AttachPainter(this);

	CComQIPtr<IStdNode>(aSpace)->AddChild(CComQIPtr<IStdNode>(m_pVolObject));
	m_pVolObject->Release();
}

/////////////////////////////////////////////////////////////////////////////
GlFloor::~GlFloor()
{
	CScopeLock sl(&m_Lock);

	m_pCoord->Release();

	if (m_bstrText)
	{
		SysFreeString(m_bstrText);
		m_bstrText = NULL;
	}

	// m_pVolObject->Release();
}

//******************************** Draw *************************************
/////////////////////////////////////////////////////////////////////////////
STDMETHODIMP GlFloor::Draw(IGlCanvas* aCanvas,
						   IGlContext* aContext, 
						   IGlViewer* aViewer,
						   IUnknown*)
{
	boolean bValidList;
	aCanvas->IsCompiledList(m_iList, &bValidList);

	if (!bValidList)
	{
		aCanvas->BeginCompileList(&m_iList);

		#define ANGLE_STEP_RAD  PI/16.0        // generate 32 faces per circle
		#define FULL_CIRCLE_RAD 2.0*PI-0.00001 // less than 2 pi, to avoid a roundoff problem

		for (double angleZ = 0.0; angleZ < 0.5 * FULL_CIRCLE_RAD; angleZ += ANGLE_STEP_RAD)
		{
			double z1        = cos(angleZ);
			double dCoeffXY1 = sin(angleZ);
			double z2        = cos(angleZ + ANGLE_STEP_RAD);
			double dCoeffXY2 = sin(angleZ + ANGLE_STEP_RAD);

			for (double angle = 0.0; angle < FULL_CIRCLE_RAD; angle += ANGLE_STEP_RAD)
			{
				double x1 = cos(angle);
				double y1 = sin(angle);
				double x2 = cos(angle + ANGLE_STEP_RAD);
				double y2 = sin(angle + ANGLE_STEP_RAD);

				GlPoint A(dCoeffXY1 * x1, dCoeffXY1 * y1, z1);
				GlPoint B(dCoeffXY2 * x1, dCoeffXY2 * y1, z2);
				GlPoint C(dCoeffXY2 * x2, dCoeffXY2 * y2, z2);
				GlPoint D(dCoeffXY1 * x2, dCoeffXY1 * y2, z1);

				aCanvas->BeginDrawPolygon();
				{
					if (z1 == 1.0)
					{
						GlPlane plane(A, B, C);
						aCanvas->SetVertexNormal(plane.GetNormal());
					}
					else
					{
						GlPlane plane(D, A, B);
						aCanvas->SetVertexNormal(plane.GetNormal());
					}

					aCanvas->Vertex(A);
					aCanvas->Vertex(B);
					aCanvas->Vertex(C);
					aCanvas->Vertex(D);
				}
				aCanvas->EndDrawVertices();
			}
		}

		aCanvas->EndCompileList();
	}

	CScopeLock sl(&m_Lock);

	aCanvas->PushModelviewMatrix(FALSE);

	aCanvas->SetVertexColor(m_Color);
	aCanvas->Scale(GlPoint(m_dRadius, m_dRadius, m_dRadius));
	aCanvas->DrawCompiledList(m_iList);

	aCanvas->PopModelviewMatrix();

	if (m_bShowText)
	{
		// Draw the text

		if (m_bstrText)
		{
			// Get the font
			IGlFont* pFont;
			aContext->GetDefaultFont(&pFont);

			if (pFont)
			{
				GlPoint refPnt(0.0, 0.0, 0.0);
				m_pVolObject->PointLCStoWCS(refPnt);
				aViewer->PointWCStoSCS(refPnt);

				refPnt.m.x += 1.0;
				aViewer->PointSCStoWCS(refPnt);
				m_pVolObject->PointWCStoLCS(refPnt);
				
				refPnt.SetMagnitude(m_dRadius);

				m_pVolObject->PointLCStoWCS(refPnt);
				aViewer->PointWCStoSCS(refPnt);
				refPnt.m.x += 0.01; // move right at 1/100 of the screen

				if (refPnt.m.z < 0.0) // the text is located infront of the observer
				{
					aCanvas->PushLightAttribute();
					aCanvas->PushProjectionMatrix(TRUE, NULL);
					aCanvas->PushModelviewMatrix(TRUE);

					aCanvas->SetVertexColor(GlColor(0.2f, 0.6f, 0.2f));
					pFont->Print(m_bstrText, refPnt);

					aCanvas->PopModelviewMatrix();
					aCanvas->PopProjectionMatrix();
					aCanvas->PopLightAttribute();
				}

				pFont->Release();
			}
		}
	}

	return S_OK;
}

/////////////////////////////////////////////////////////////////////////////
void GlFloor::SetAbsDisp(double aDisp)
{
	CScopeLock sl(&m_Lock);
	m_pCoord->SetOrigin3d(aDisp, m_iFloor * GetFloorHeight(), 0.0);
}

//=================================== EOF ===================================
