/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			IsoSystem.cpp
//		DESCRIPTION:	implementation of the CIsoSystem class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "epet.h"
#include "IsoSystem.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define MAX_RIGIDITY 100000000.0
#define MIN_DAMPING 0.0

//***************************************************************************
//                            CIsoSystem CLASS
//***************************************************************************

/////////////////////////////////////////////////////////////////////////////
CIsoSystem::CIsoSystem(long aID) :
m_lID(aID),
m_eType(TYPE_UNDEFINED),
m_strDescription("")
{
}

/////////////////////////////////////////////////////////////////////////////
CIsoSystem::~CIsoSystem()
{
}

/////////////////////////////////////////////////////////////////////////////
CString CIsoSystem::GetNameByType(EType aType)
{
	switch (aType)
	{
	case CIsoSystem::TYPE_AFMS:      return "AFMS System";            
	case CIsoSystem::TYPE_AFMS_CD:   return "AFMS CD System";         
	case CIsoSystem::TYPE_SLIDING:   return "Earthquake Protector";         
	case CIsoSystem::TYPE_SHEAR_VIS: return "Shear Viscous";   
	case CIsoSystem::TYPE_SHEAR_HYS: return "Shear Hysteretic";
	case CIsoSystem::TYPE_FIXED:     return "Fixed System";           
	default:                         return "Unknown system";         
	}
}

//***************************************************************************
//                         CIsoSystemFixed CLASS
//***************************************************************************

/////////////////////////////////////////////////////////////////////////////
CIsoSystemFixed::CIsoSystemFixed() :
CIsoSystem()
{
	m_eType = TYPE_FIXED;
	m_strDescription = "Fixed system (no isolation)";
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemFixed::Save() 
{ 
	return true; 
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemFixed::IsValid() const 
{ 
	return true; 
}

/////////////////////////////////////////////////////////////////////////////
CString CIsoSystemFixed::GetParamDesc() const 
{ 
	return m_strDescription; 
}

/////////////////////////////////////////////////////////////////////////////
double CIsoSystemFixed::GetRigidity_KG_SEC2(double aBasementMass, 
											double aTotalMass, 
											double aTime, 
											double aRelDisp, 
											double aRelVel) const
{
	// Max rigidity
	return MAX_RIGIDITY;
}

/////////////////////////////////////////////////////////////////////////////
double CIsoSystemFixed::GetDamping_KG_SEC(double aBasementMass, 
										  double aTotalMass, 
										  double aTime, 
										  double aRelDisp, 
										  double aRelVel) const
{
	// Min damping
	return MIN_DAMPING;
}

/////////////////////////////////////////////////////////////////////////////
double CIsoSystemFixed::GetMaxRelDisp_CM() const
{
	// Fixed system - no relative displacement
	return 0.0;
}

//***************************************************************************
//                          CIsoSystemAFMS CLASS
//***************************************************************************

/////////////////////////////////////////////////////////////////////////////
CIsoSystemAFMS::CIsoSystemAFMS(long aID) :
CIsoSystem(aID),
m_dNaturalFriction(0.02),
m_dCoeffN(2.0)
{
	if (m_lID)
	{
		IDbTable* pTable = g_Application.GetDatabase()->GetIsoSysAFMSTable();
		IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

		long lFieldID;

		BSTR bstrDesc;
		pTable->GetFieldIDByName(A2BSTR("DESCRIPTION"), &lFieldID);
		pRS->GetFieldAsString(m_lID, lFieldID, &bstrDesc);

		if (bstrDesc)
		{
			m_strDescription = bstrDesc;
			SysFreeString(bstrDesc);
		}

		long lType;
		pTable->GetFieldIDByName(A2BSTR("ISO_TYPE"), &lFieldID);
		pRS->GetFieldAsInteger(m_lID, lFieldID, &lType);
		m_eType = (EType)lType;

		pTable->GetFieldIDByName(A2BSTR("NATURAL_FRICTION"), &lFieldID);
		pRS->GetFieldAsFloat(m_lID, lFieldID, &m_dNaturalFriction);

		pTable->GetFieldIDByName(A2BSTR("COEFF_N"), &lFieldID);
		pRS->GetFieldAsFloat(m_lID, lFieldID, &m_dCoeffN);

		for (int i = 0; i < MAX_VRADS; i++)
		{
			double dValue;

			CString str;
			str.Format("VRAD%u_CM", i);
			pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
			pRS->GetFieldAsFloat(m_lID, lFieldID, &dValue);

			if (dValue)
			{
				m_adVRads.push_back(dValue);
			}
			else
			{
				break;
			}
		}

		for (i = 0; i < MAX_HRADS; i++)
		{
			double dValue;

			CString str;
			str.Format("HRAD%u_CM", i);
			pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
			pRS->GetFieldAsFloat(m_lID, lFieldID, &dValue);

			if (dValue)
			{
				m_adHRads.push_back(dValue);
			}
			else
			{
				break;
			}
		}
	}
	else
	{
		m_eType = TYPE_AFMS;

		m_adVRads.push_back(100.0);
		m_adVRads.push_back(200.0);

		m_adHRads.push_back(2.5);
		m_adHRads.push_back(5.0);
	}
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemAFMS::Save()
{
	if (!IsValid())
	{
		return false;
	}

	IDbTable* pTable = g_Application.GetDatabase()->GetIsoSysAFMSTable();
	IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

	if (!m_lID)
	{
		// Create record
		pRS->AddRecord(DB_PRIMARY_KEY_AUTO_GENERATE, &m_lID);
		ASSERT(m_lID);
	}

	long lFieldID;
	pTable->GetFieldIDByName(A2BSTR("DESCRIPTION"), &lFieldID);
	pRS->SetFieldToString(m_lID, lFieldID, m_strDescription.IsEmpty() ? NULL : A2BSTR(m_strDescription));

	pTable->GetFieldIDByName(A2BSTR("ISO_TYPE"), &lFieldID);
	pRS->SetFieldToInteger(m_lID, lFieldID, m_eType);

	pTable->GetFieldIDByName(A2BSTR("NATURAL_FRICTION"), &lFieldID);
	pRS->SetFieldToFloat(m_lID, lFieldID, m_dNaturalFriction);

	pTable->GetFieldIDByName(A2BSTR("COEFF_N"), &lFieldID);
	pRS->SetFieldToFloat(m_lID, lFieldID, m_dCoeffN);

	// Write VRads ----------------------------------------------------------

	for (int i = 0, nRads = m_adVRads.size(); i < nRads; i++)
	{
		if (!m_adVRads[i])
		{
			break;
		}

		CString str;
		str.Format("VRAD%u_CM", i);
		pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
		pRS->SetFieldToFloat(m_lID, lFieldID, m_adVRads[i]);
	}

	for (; i < MAX_VRADS; i++)
	{
		CString str;
		str.Format("VRAD%u_CM", i);
		pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
		pRS->SetFieldToFloat(m_lID, lFieldID, 0.0);
	}

	// Write HRads ----------------------------------------------------------

	for (i = 0, nRads = m_adHRads.size(); i < nRads; i++)
	{
		if (!m_adHRads[i])
		{
			break;
		}

		CString str;
		str.Format("HRAD%u_CM", i);
		pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
		pRS->SetFieldToFloat(m_lID, lFieldID, m_adHRads[i]);
	}

	for (; i < MAX_HRADS; i++)
	{
		CString str;
		str.Format("HRAD%u_CM", i);
		pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
		pRS->SetFieldToFloat(m_lID, lFieldID, 0.0);
	}

	g_Application.GetDatabase()->Commit();

	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemAFMS::IsValid() const
{
	return (m_dNaturalFriction > 0.0 && m_dCoeffN > 0.0 && m_adVRads.size() && m_adHRads.size());
}

/////////////////////////////////////////////////////////////////////////////
CString CIsoSystemAFMS::GetParamDesc() const
{
	CString str;
	str.Format(
		"Type: %s; Natural friction: %.4f; Coeff-N: %.2f", 
		(m_eType == TYPE_AFMS) ? "AFMS" : "AFMS-CD",
		m_dNaturalFriction, 
		m_dCoeffN);

	return str;
}

/////////////////////////////////////////////////////////////////////////////
double CIsoSystemAFMS::GetRigidity_KG_SEC2(double aBasementMass, 
										   double aTotalMass, 
										   double aTime, 
										   double aRelDisp, 
										   double aRelVel) const
{
	ASSERT(m_adHRads.size() && m_adVRads.size());

	aRelDisp = fabs(aRelDisp);
	double dRadius = 0.0;

	if (m_eType == TYPE_AFMS_CD && aRelDisp < m_adHRads[0])
	{
		// Consider central depression
		double dVR = 0.16 * pow(aTotalMass, 0.4);
		double dHR = dVR * m_dNaturalFriction * m_dCoeffN;

		if (aRelDisp < dHR)
		{
			dRadius = dVR;
		}
	}

	if (!dRadius)
	{
		for (int i = 0, nCount = m_adHRads.size(); i < nCount; i++)
		{
			if (aRelDisp < m_adHRads[i])
			{
				dRadius = m_adVRads[i];
				break;
			}
		}
	}

	if (aRelDisp > dRadius)
		return MAX_RIGIDITY;

	double dRet = 981.0 * aTotalMass / dRadius;
	return dRet;
}

/////////////////////////////////////////////////////////////////////////////
double CIsoSystemAFMS::GetDamping_KG_SEC(double aBasementMass, 
									     double aTotalMass, 
										 double aTime, 
										 double aRelDisp, 
										 double aRelVel) const
{
	if (!aRelVel)
		return MIN_DAMPING;

	double dRet = 981.0 * aTotalMass * m_dNaturalFriction / fabs(aRelVel);
	return dRet;
}

/////////////////////////////////////////////////////////////////////////////
double CIsoSystemAFMS::GetMaxRelDisp_CM() const
{
	return m_adHRads.back();
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemAFMS::SetTypeToAFMSCD(bool aCD) 
{ 
	m_eType = aCD ? TYPE_AFMS_CD : TYPE_AFMS; 
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemAFMS::SetNaturalFriction(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_dNaturalFriction = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemAFMS::SetCoeffN(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_dCoeffN = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemAFMS::SetVRadius_CM(int aRadIndex, double aValue)
{
	int nRads = m_adVRads.size();

	if (aValue < 0.0 || aRadIndex < 0 || aRadIndex > nRads || aRadIndex >= MAX_VRADS)
	{
		return false;
	}

	if (aValue == 0.0)
	{
		m_adVRads.resize(aRadIndex);
	}
	else if (aRadIndex == nRads)
	{
		m_adVRads.push_back(aValue);
	}
	else
	{
		m_adVRads[aRadIndex] = aValue;
	}

	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemAFMS::SetHRadius_CM(int aRadIndex, double aValue)
{
	int nRads = m_adHRads.size();

	if (aValue < 0.0 || aRadIndex < 0 || aRadIndex > nRads || aRadIndex >= MAX_HRADS)
	{
		return false;
	}

	if (aValue == 0.0)
	{
		m_adHRads.resize(aRadIndex);
	}
	else if (aRadIndex == nRads)
	{
		m_adHRads.push_back(aValue);
	}
	else
	{
		m_adHRads[aRadIndex] = aValue;
	}

	return true;
}


//***************************************************************************
//                        CIsoSystemShear CLASS
//***************************************************************************

/////////////////////////////////////////////////////////////////////////////
CIsoSystemShear::CIsoSystemShear(long aID) :
CIsoSystem(aID),
m_dDampingRatio(0.2),
m_dCoeffA(69.7),
m_dCoeffB(0.3),
m_dMaxRelDisp(0.0),
m_dMaxRelVel(0.0)
{
	if (m_lID)
	{
		IDbTable* pTable = g_Application.GetDatabase()->GetIsoSysShearTable();
		IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

		long lFieldID;

		BSTR bstrDesc;
		pTable->GetFieldIDByName(A2BSTR("DESCRIPTION"), &lFieldID);
		pRS->GetFieldAsString(m_lID, lFieldID, &bstrDesc);

		if (bstrDesc)
		{
			m_strDescription = bstrDesc;
			SysFreeString(bstrDesc);
		}

		long lType;
		pTable->GetFieldIDByName(A2BSTR("ISO_TYPE"), &lFieldID);
		pRS->GetFieldAsInteger(m_lID, lFieldID, &lType);
		m_eType = (EType)lType;

		pTable->GetFieldIDByName(A2BSTR("DAMPING_RATIO"), &lFieldID);
		pRS->GetFieldAsFloat(m_lID, lFieldID, &m_dDampingRatio);

		pTable->GetFieldIDByName(A2BSTR("COEFF_A"), &lFieldID);
		pRS->GetFieldAsFloat(m_lID, lFieldID, &m_dCoeffA);

		pTable->GetFieldIDByName(A2BSTR("COEFF_B"), &lFieldID);
		pRS->GetFieldAsFloat(m_lID, lFieldID, &m_dCoeffB);
	}
	else
	{
		m_eType = TYPE_SHEAR_HYS;
	}
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemShear::Save()
{
	if (!IsValid())
	{
		return false;
	}

	IDbTable* pTable = g_Application.GetDatabase()->GetIsoSysShearTable();
	IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

	if (!m_lID)
	{
		// Create record
		pRS->AddRecord(DB_PRIMARY_KEY_AUTO_GENERATE, &m_lID);
		ASSERT(m_lID);
	}

	long lFieldID;
	pTable->GetFieldIDByName(A2BSTR("DESCRIPTION"), &lFieldID);
	pRS->SetFieldToString(m_lID, lFieldID, m_strDescription.IsEmpty() ? NULL : A2BSTR(m_strDescription));

	pTable->GetFieldIDByName(A2BSTR("ISO_TYPE"), &lFieldID);
	pRS->SetFieldToInteger(m_lID, lFieldID, m_eType);

	pTable->GetFieldIDByName(A2BSTR("DAMPING_RATIO"), &lFieldID);
	pRS->SetFieldToFloat(m_lID, lFieldID, m_dDampingRatio);

	pTable->GetFieldIDByName(A2BSTR("COEFF_A"), &lFieldID);
	pRS->SetFieldToFloat(m_lID, lFieldID, m_dCoeffA);

	pTable->GetFieldIDByName(A2BSTR("COEFF_B"), &lFieldID);
	pRS->SetFieldToFloat(m_lID, lFieldID, m_dCoeffB);

	g_Application.GetDatabase()->Commit();

	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemShear::IsValid() const
{
	return (m_dDampingRatio > 0.0 && m_dCoeffA > 0.0 && m_dCoeffB > 0.0);
}

/////////////////////////////////////////////////////////////////////////////
CString CIsoSystemShear::GetParamDesc() const
{
	CString str;
	str.Format(
		"Type: %s; Damping ratio: %.2f; Coeff-A: %.2f; Coeff-B: %.2f", 
		(m_eType == TYPE_SHEAR_HYS) ? "Shear Hys" : "Shear Vis",
		m_dDampingRatio, 
		m_dCoeffA, 
		m_dCoeffB);

	return str;
}

/////////////////////////////////////////////////////////////////////////////
double CIsoSystemShear::GetRigidity_KG_SEC2(double aBasementMass, 
											double aTotalMass, 
											double aTime, 
											double aRelDisp, 
											double aRelVel) const
{
	double dRet = aTotalMass * m_dCoeffA * exp(-m_dCoeffB * sqrt(min(fabs(aRelDisp), 37.5)));
	return dRet;
}

/////////////////////////////////////////////////////////////////////////////
double CIsoSystemShear::GetDamping_KG_SEC(double aBasementMass, 
										  double aTotalMass, 
										  double aTime, 
										  double aRelDisp, 
										  double aRelVel) const
{
	if (!aRelDisp && !aRelVel)
	{
		m_dMaxRelDisp = 0.0;
		m_dMaxRelVel = 0.0;
	}

	double dRet(MIN_DAMPING);
	
	if (m_eType == TYPE_SHEAR_VIS)
	{
		if (m_dMaxRelVel)
		{
			dRet = 2.0 * m_dDampingRatio * aBasementMass * m_dMaxRelVel / m_dMaxRelDisp;
		}
	}
	else // hysteretic
	{
		if (aRelVel)
		{
			dRet = GetRigidity_KG_SEC2(aBasementMass, aTotalMass, aTime, aRelDisp, aRelVel) *
			       2.0 * m_dDampingRatio * fabs(aRelDisp / aRelVel);
		}
	}

	m_dMaxRelDisp = max(m_dMaxRelDisp, fabs(aRelDisp));
	m_dMaxRelVel  = max(m_dMaxRelDisp, fabs(aRelVel));

	return dRet;
}

/////////////////////////////////////////////////////////////////////////////
double CIsoSystemShear::GetMaxRelDisp_CM() const
{
	return 100.0;
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemShear::SetTypeToHysteretic(bool aHys)
{
	m_eType = aHys ? TYPE_SHEAR_HYS : TYPE_SHEAR_VIS;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemShear::SetDampingRatio(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_dDampingRatio = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemShear::SetCoeffA(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_dCoeffA = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemShear::SetCoeffB(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_dCoeffB = aValue;
	return true;
}


//***************************************************************************
//                        CIsoSystemSliding CLASS
//***************************************************************************

/////////////////////////////////////////////////////////////////////////////
CIsoSystemSliding::CIsoSystemSliding(long aID) :
CIsoSystem(aID),
m_dNaturalFriction(0.1),
m_dRadius_CM(300.0)
{
	m_eType = TYPE_SLIDING;

	if (m_lID)
	{
		IDbTable* pTable = g_Application.GetDatabase()->GetIsoSysSlidingTable();
		IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

		long lFieldID;

		BSTR bstrDesc;
		pTable->GetFieldIDByName(A2BSTR("DESCRIPTION"), &lFieldID);
		pRS->GetFieldAsString(m_lID, lFieldID, &bstrDesc);

		if (bstrDesc)
		{
			m_strDescription = bstrDesc;
			SysFreeString(bstrDesc);
		}

		pTable->GetFieldIDByName(A2BSTR("NATURAL_FRICTION"), &lFieldID);
		pRS->GetFieldAsFloat(m_lID, lFieldID, &m_dNaturalFriction);

		pTable->GetFieldIDByName(A2BSTR("RAD_CM"), &lFieldID);
		pRS->GetFieldAsFloat(m_lID, lFieldID, &m_dRadius_CM);
	}
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemSliding::Save()
{
	if (!IsValid())
	{
		return false;
	}

	IDbTable* pTable = g_Application.GetDatabase()->GetIsoSysSlidingTable();
	IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

	if (!m_lID)
	{
		// Create record
		pRS->AddRecord(DB_PRIMARY_KEY_AUTO_GENERATE, &m_lID);
		ASSERT(m_lID);
	}

	long lFieldID;
	pTable->GetFieldIDByName(A2BSTR("DESCRIPTION"), &lFieldID);
	pRS->SetFieldToString(m_lID, lFieldID, m_strDescription.IsEmpty() ? NULL : A2BSTR(m_strDescription));

	pTable->GetFieldIDByName(A2BSTR("NATURAL_FRICTION"), &lFieldID);
	pRS->SetFieldToFloat(m_lID, lFieldID, m_dNaturalFriction);

	pTable->GetFieldIDByName(A2BSTR("RAD_CM"), &lFieldID);
	pRS->SetFieldToFloat(m_lID, lFieldID, m_dRadius_CM);

	g_Application.GetDatabase()->Commit();

	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemSliding::IsValid() const
{
	return (m_dNaturalFriction > 0.0 && m_dRadius_CM > 0.0);
}

/////////////////////////////////////////////////////////////////////////////
CString CIsoSystemSliding::GetParamDesc() const
{
	CString str;
	str.Format(
		"Type: Earthquake Protector; Natural friction: %.4f; Radius: %.2f cm", 
		m_dNaturalFriction, 
		m_dRadius_CM);

	return str;
}

/////////////////////////////////////////////////////////////////////////////
double CIsoSystemSliding::GetRigidity_KG_SEC2(double aBasementMass, 
											  double aTotalMass, 
											  double aTime, 
											  double aRelDisp, 
											  double aRelVel) const
{
	aRelDisp = fabs(aRelDisp);

	if (aRelDisp > m_dRadius_CM)
		return MAX_RIGIDITY;

	double dRet = 981.0 * aTotalMass / m_dRadius_CM;
	return dRet;
}

/////////////////////////////////////////////////////////////////////////////
double CIsoSystemSliding::GetDamping_KG_SEC(double aBasementMass, 
										    double aTotalMass, 
										    double aTime, 
										    double aRelDisp, 
										    double aRelVel) const
{
	if (!aRelVel)
		return MIN_DAMPING;

	double dRet = 981.0 * aTotalMass * m_dNaturalFriction / fabs(aRelVel);
	return dRet;
}

/////////////////////////////////////////////////////////////////////////////
double CIsoSystemSliding::GetMaxRelDisp_CM() const
{
	return 0.6 * m_dRadius_CM;
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemSliding::SetNaturalFriction(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_dNaturalFriction = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CIsoSystemSliding::SetRadius_CM(double aValue)
{
	if (aValue <= 0.0)
	{
		return false;
	}

	m_dRadius_CM = aValue;
	return true;
}

//=================================== EOF ===================================
