/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			IsoSystem.h
//		DESCRIPTION:	interface for the CIsoSystem class
//
/////////////////////////////////////////////////////////////////////////////


#if !defined(AFX_ISOSYSTEM_H__9437334A_AB13_426F_953F_3674C23C0766__INCLUDED_)
#define AFX_ISOSYSTEM_H__9437334A_AB13_426F_953F_3674C23C0766__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


//***************************************************************************
//                            CIsoSystem CLASS
//***************************************************************************

class CIsoSystem  
{
public:

	enum EType
	{
		TYPE_UNDEFINED = 0,
		TYPE_FIXED     = TYPE_UNDEFINED,
		TYPE_AFMS      = 1,
		TYPE_AFMS_CD   = 2,
		TYPE_SLIDING   = 3,
		TYPE_SHEAR_VIS = 4,
		TYPE_SHEAR_HYS = 5,
	};

protected:

	long m_lID;
	EType m_eType;
	CString m_strDescription;

public:

	CIsoSystem(long aID = ID_NEW_RECORD);
	virtual ~CIsoSystem();

	virtual bool Save() = 0;
	virtual bool IsValid() const = 0;
	virtual CString GetParamDesc() const = 0;

	virtual double GetRigidity_KG_SEC2(double aBasementMass, double aTotalMass, double aTime, double aRelDisp, double aRelVel) const = 0;
	virtual double GetDamping_KG_SEC(double aBasementMass, double aTotalMass, double aTime, double aRelDisp, double aRelVel) const = 0;
	virtual double GetMaxRelDisp_CM() const = 0;

	inline long GetID() const { return m_lID; }
	inline EType GetType() const { return m_eType; }

	inline const char* GetDescription() const { return m_strDescription; }
	inline bool SetDescription(const char* aValue) { m_strDescription = aValue; return true; }

	static CString GetNameByType(EType aType);
};


//***************************************************************************
//                         CIsoSystemFixed CLASS
//***************************************************************************

class CIsoSystemFixed : public CIsoSystem
{
public:

	CIsoSystemFixed();

	virtual bool Save();
	virtual bool IsValid() const;
	virtual CString GetParamDesc() const;

	virtual double GetRigidity_KG_SEC2(double aBasementMass, double aTotalMass, double aTime, double aRelDisp, double aRelVel) const;
	virtual double GetDamping_KG_SEC(double aBasementMass, double aTotalMass, double aTime, double aRelDisp, double aRelVel) const;
	virtual double GetMaxRelDisp_CM() const;
};


//***************************************************************************
//                          CIsoSystemAFMS CLASS
//***************************************************************************

#define MAX_VRADS 10
#define MAX_HRADS 10

class CIsoSystemAFMS : public CIsoSystem
{
protected:

	double m_dNaturalFriction;
	double m_dCoeffN;
	vector<double> m_adVRads;
	vector<double> m_adHRads;

public:

	CIsoSystemAFMS(long aID = ID_NEW_RECORD);

	// Archiving ------------------------------------------------------------

	virtual bool Save();
	virtual bool IsValid() const;
	virtual CString GetParamDesc() const;

	// Get data -------------------------------------------------------------

	virtual double GetRigidity_KG_SEC2(double aBasementMass, double aTotalMass, double aTime, double aRelDisp, double aRelVel) const;
	virtual double GetDamping_KG_SEC(double aBasementMass, double aTotalMass, double aTime, double aRelDisp, double aRelVel) const;
	virtual double GetMaxRelDisp_CM() const;

	inline double GetNaturalFriction() const { return m_dNaturalFriction; }
	inline double GetCoeffN() const { return m_dCoeffN; }

	inline int GetNumberOfVRadiuses() const { return m_adVRads.size(); }
	inline double GetVRadius_CM(int aIndex) const 
	{ 
		int nRads = m_adVRads.size();
		return (aIndex < 0 || aIndex >= nRads) ? 0.0 : m_adVRads.at(aIndex); 
	}

	inline int GetNumberOfHRadiuses() const { return m_adHRads.size(); }
	inline double GetHRadius_CM(int aIndex) const 
	{ 
		int nRads = m_adHRads.size();
		return (aIndex < 0 || aIndex >= nRads) ? 0.0 : m_adHRads.at(aIndex); 
	}

	// Modifying data -------------------------------------------------------

	bool SetTypeToAFMSCD(bool aCD);
	bool SetNaturalFriction(double aValue);
	bool SetCoeffN(double aValue);
	bool SetVRadius_CM(int aIndex, double aValue);
	bool SetHRadius_CM(int aIndex, double aValue);
};


//***************************************************************************
//                        CIsoSystemShear CLASS
//***************************************************************************

class CIsoSystemShear : public CIsoSystem
{
	double m_dDampingRatio;
	double m_dCoeffA;
	double m_dCoeffB;

	double mutable m_dMaxRelDisp;
	double mutable m_dMaxRelVel;

public:

	CIsoSystemShear(long aID = ID_NEW_RECORD);

	virtual bool Save();
	virtual bool IsValid() const;
	virtual CString GetParamDesc() const;

	virtual double GetRigidity_KG_SEC2(double aBasementMass, double aTotalMass, double aTime, double aRelDisp, double aRelVel) const;
	virtual double GetDamping_KG_SEC(double aBasementMass, double aTotalMass, double aTime, double aRelDisp, double aRelVel) const;
	virtual double GetMaxRelDisp_CM() const;

	inline double GetDampingRatio() const { return m_dDampingRatio; }
	inline double GetCoeffA() const { return m_dCoeffA; }
	inline double GetCoeffB() const { return m_dCoeffB; }

	bool SetTypeToHysteretic(bool aHys);
	bool SetDampingRatio(double aValue);
	bool SetCoeffA(double aValue);
	bool SetCoeffB(double aValue);
};


//***************************************************************************
//                        CIsoSystemSliding CLASS
//***************************************************************************

class CIsoSystemSliding : public CIsoSystem
{
	double m_dNaturalFriction;
	double m_dRadius_CM;

public:

	CIsoSystemSliding(long aID = ID_NEW_RECORD);

	virtual bool Save();
	virtual bool IsValid() const;
	virtual CString GetParamDesc() const;

	virtual double GetRigidity_KG_SEC2(double aBasementMass, double aTotalMass, double aTime, double aRelDisp, double aRelVel) const;
	virtual double GetDamping_KG_SEC(double aBasementMass, double aTotalMass, double aTime, double aRelDisp, double aRelVel) const;
	virtual double GetMaxRelDisp_CM() const;

	inline double GetNaturalFriction() const { return m_dNaturalFriction; }
	inline double GetRadius_CM() const { return m_dRadius_CM; }

	bool SetNaturalFriction(double aValue);
	bool SetRadius_CM(double aValue);
};


#endif // !defined(AFX_ISOSYSTEM_H__9437334A_AB13_426F_953F_3674C23C0766__INCLUDED_)

//=================================== EOF ===================================
